//////////////////////////////////////////////////////////////////
// Symbolic
//
// Library for the construction of nested symbolic propositions
// The Flying Pig!
// Started 5/8/2003
// http://www.flypig.co.uk
// 
// 
// The MIT License
// See symbolic.h, COPYING file or website for licence
// 
//////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////
// Includes

#include "symbolic.h"
#include "symbolic_private.h"

#include <stdio.h>
#include <float.h>
#define _USE_MATH_DEFINES
#include <math.h>

//////////////////////////////////////////////////////////////////
// Global variables

//////////////////////////////////////////////////////////////////
// Main application

//////////////////////////////////////////////////////////////////
// Main program
int main (int argc, char * * argv) {
  Operation     *psOp;
  Operation     *psOp2;
  char          szString[512];
  Operation     *psSub;
  Operation     *psFind;
  Operation     *psFind2;
  double        fResult;
  Variable      *psVars = NULL;
  Variable      *psVar = NULL;

  argc = argc;
  argv = argv;

  printf ("Example structure\n");

  PropMemReset ();

  psOp = CreateBinary (OPBINARY_LOR, CreateBinary (OPBINARY_LAND,
    CreateVariable ("m1"), CreateVariable ("m2")),
    CreateBinary (OPBINARY_IND, CreateMemory (), CreateInteger (57)));
  psOp2 = CopyRecursive (psOp);
  psFind = CreateBinary (OPBINARY_LAND, CreateVariable ("m1"),
    CreateVariable ("m2"));
  psFind2 = CreateVariable ("m2");
  psSub = CreateBinary (OPBINARY_SUB, CreateVariable ("m2"),
    CreateVariable ("m2"));
  psSub = SubstituteOperation (psSub, psFind2, psFind);

  OperationToString (psOp2, szString, sizeof(szString));
  printf("Copied is: %s\n", szString);

  OperationToString (psOp, szString, sizeof(szString));
  printf("Output is: %s\n", szString);

  psOp = SubstituteOperation (psOp, psFind, psSub);
  psOp = SubstituteOperation (psOp, psFind, psSub);

  OperationToString (psFind, szString, sizeof(szString));
  printf("To find is: %s\n", szString);

  OperationToString (psSub, szString, sizeof(szString));
  printf("To substitute is: %s\n", szString);

  OperationToString (psOp, szString, sizeof(szString));
  printf("Substituted twice is: %s\n", szString);

  FreeRecursive (psOp);
  FreeRecursive (psOp2);
  FreeRecursive (psFind);
  FreeRecursive (psFind2);
  FreeRecursive (psSub);

  psOp = StringToOperation ("((m1 ^ m2) v (M ind 57))");
  psOp2 = CopyRecursive (psOp);
  psFind = StringToOperation ("(m1 ^ m2)");
  psFind2 = StringToOperation ("m2");
  psSub = StringToOperation ("(m2 - m2)");
  psSub = SubstituteOperation (psSub, psFind2, psFind);

  OperationToString (psOp2, szString, sizeof(szString));
  printf("Copied is: %s\n", szString);

  OperationToString (psOp, szString, sizeof(szString));
  printf("Output is: %s\n", szString);

  psOp = SubstituteOperation (psOp, psFind, psSub);
  psOp = SubstituteOperation (psOp, psFind, psSub);

  OperationToString (psFind, szString, sizeof(szString));
  printf("To find is: %s\n", szString);

  OperationToString (psSub, szString, sizeof(szString));
  printf("To substitute is: %s\n", szString);

  OperationToString (psOp, szString, sizeof(szString));
  printf("Substituted twice is: %s\n", szString);

  fResult = ApproximateOperation (psOp);
  printf("Approximates to: %f\n", fResult);

  FreeRecursive (psOp);
  FreeRecursive (psOp2);
  FreeRecursive (psFind);
  FreeRecursive (psFind2);
  FreeRecursive (psSub);

  PropMemOutput ();

  printf ("\n");

  printf ("Example formula\n");
  psOp = StringToOperation ("2**(8/(2**2))");
  OperationToString (psOp, szString, sizeof(szString));
  printf("Formula is: %s\n", szString);
  fResult = ApproximateOperation (psOp);
  printf("Approximates to: %f\n", fResult);
  psOp = UberSimplifyOperation (psOp);
  OperationToString (psOp, szString, sizeof(szString));
  printf("Simplified is: %s\n", szString);
  fResult = ApproximateOperation (psOp);
  printf("Approximates to: %f\n", fResult);

  FreeRecursive (psOp);

  PropMemOutput ();

  printf ("\n");

  printf ("Example continued fraction\n");
  psOp = ContinuedFraction (0.398472398472, CONTFRAC_MAXDEPTH);
  OperationToString (psOp, szString, sizeof(szString));
  printf("Continued fraction is: %s\n", szString);
  psOp = UberSimplifyOperation (psOp);
  OperationToString (psOp, szString, sizeof(szString));
  printf("Simplified fraction is: %s\n", szString);
  fResult = ApproximateOperation (psOp);
  printf("Approximates to: %f\n", fResult);

  FreeRecursive (psOp);

  PropMemOutput ();

  printf ("\n");

  printf ("Example derivative\n");
  psOp = StringToOperation ("cos ((x**2) + (y**2)) / ((x**2) + (y**2))");
  //psOp = StringToOperation ("(x**2) + (y**2)");
  OperationToString (psOp, szString, sizeof(szString));
  printf("Formula is: %s\n", szString);

  psVars = CreateVariables (psOp, psVars);
  printf("Number of variables: %d\n", VariableCount (psVars));
  psVar = FindVariable (psVars, "x");
  SetVariable (psVar, 2.0);
  psVar = FindVariable (psVars, "y");
  SetVariable (psVar, 3.0);
  psVar = FindVariable (psVars, "x");
  printf ("Variable x has value %f\n", GetVariable (psVar));
  psVar = FindVariable (psVars, "y");
  printf ("Variable y has value %f\n", GetVariable (psVar));
  fResult = ApproximateOperation (psOp);
  printf ("Approximates to: %f\n", fResult);
  psVars = FreeVariables (psVars);
  printf("Number of variables: %d\n", VariableCount (psVars));

  psSub = StringToOperation ("x");
  OperationToString (psSub, szString, sizeof(szString));
  printf("Differentiate WRT: %s\n", szString);
  psOp2 = DifferentiateOperation (psOp, psSub);
  OperationToString (psOp2, szString, sizeof(szString));
  printf("Derivative is: %s\n", szString);

  psVars = CreateVariables (psOp2, psVars);
  fResult = ApproximateOperation (psOp2);
  printf ("Approximates to: %f\n", fResult);

  psOp2 = UberSimplifyOperation (psOp2);
  OperationToString (psOp2, szString, sizeof(szString));
  printf("Simplified is: %s\n", szString);

  psVars = CreateVariables (psOp2, psVars);
  fResult = ApproximateOperation (psOp2);
  printf ("Approximates to: %f\n", fResult);

  FreeRecursive (psSub);
  FreeRecursive (psOp2);
  FreeRecursive (psOp);

  psVars = FreeVariables (psVars);
  printf("Number of variables: %d\n", VariableCount (psVars));

  PropMemOutput ();

  printf ("\n");

  return 0;
}

